/*
 * Copyright (C) 2006, 2010  Novell, Inc.
 * Copyright (C) 2015  Red Hat, Inc.
 * Written by Andreas Gruenbacher <agruen@kernel.org>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __RICHACL_XATTR_H
#define __RICHACL_XATTR_H

#include <linux/richacl.h>

#define RICHACL_XATTR "system.richacl"

struct richace_xattr {
	__le16		e_type;
	__le16		e_flags;
	__le32		e_mask;
	__le32		e_id;
};

struct richacl_xattr {
	unsigned char	a_version;
	unsigned char	a_flags;
	__le16		a_unused;
	__le32		a_owner_mask;
	__le32		a_group_mask;
	__le32		a_other_mask;
};

#define RICHACL_XATTR_VERSION	0
#define RICHACL_XATTR_MAX_COUNT \
	((XATTR_SIZE_MAX - sizeof(struct richacl_xattr)) / \
	 sizeof(struct richace_xattr))

extern struct richacl *richacl_from_xattr(struct user_namespace *, const void *, size_t);
extern size_t richacl_xattr_size(const struct richacl *);
extern int richacl_to_xattr(struct user_namespace *, const struct richacl *, void *, size_t);

extern void richacl_fix_xattr_from_user(void *, size_t);
extern void richacl_fix_xattr_to_user(void *, size_t);

#endif /* __RICHACL_XATTR_H */
